@@ -13,7 +13,7 @@ $Data::Dumper::Indent = 1;
 
 use vars qw($VERSION %IRSSI);
 
-$VERSION = "2.4.1";
+$VERSION = "2.4.4";
 %IRSSI   = (
     authors     => 'Dan Boger',
     contact     => 'zigdon@gmail.com',
@@ -89,6 +89,10 @@ sub cmd_direct_as {
 
     return unless $username = &valid_username($username);
 
+    if (!utf8::is_utf8($text)) {
+        $text = decode(&get_charset, $text);
+    }
+
     eval {
         if ( $twits{$username}
             ->new_direct_message( { user => $target, text => $text } ) )
@@ -131,11 +135,6 @@ sub cmd_retweet {
 sub cmd_retweet_as {
     my ( $data, $server, $win ) = @_;
 
-    unless ( Irssi::settings_get_bool("twirssi_track_replies") ) {
-        &notice("twirssi_track_replies is required in order to reteet.");
-        return;
-    }
-
     return unless &logged_in($twit);
 
     $data =~ s/^\s+|\s+$//;
@@ -183,6 +182,10 @@ sub cmd_retweet_as {
 
     return if $modified and &too_long($data);
 
+    if (!utf8::is_utf8($data)) {
+        $data = decode &get_charset, $data;
+    }
+
     my $success = 1;
     eval {
         if ($modified)
@@ -208,8 +211,8 @@ sub cmd_retweet_as {
         return;
     }
 
-    foreach ( $data =~ /@([-\w]+)/ ) {
-        $nicks{$1} = time;
+    foreach ( $data =~ /@([-\w]+)/g ) {
+        $nicks{$_} = time;
     }
 
     &notice("Retweet sent");
@@ -249,9 +252,14 @@ sub cmd_tweet_as {
 
     return if &too_long($data);
 
+    if (!utf8::is_utf8($data)) {
+        $data = decode &get_charset, $data;
+    }
+
     my $success = 1;
+    my $res;
     eval {
-        unless ( $twits{$username}->update($data) )
+        unless ( $res = $twits{$username}->update($data) )
         {
             &notice("Update failed");
             $success = 0;
@@ -264,13 +272,35 @@ sub cmd_tweet_as {
         return;
     }
 
-    foreach ( $data =~ /@([-\w]+)/ ) {
-        $nicks{$1} = time;
+    foreach ( $data =~ /@([-\w]+)/g ) {
+        $nicks{$_} = time;
     }
 
-    my $away = &update_away($data);
+    $id_map{__last_tweet}{$username} = $res->{id};
 
-    &notice( "Update sent" . ( $away ? " (and away msg set)" : "" ) );
+    if ( $username eq "$user\@$defservice" ) {
+      my $away = &update_away($data);
+
+      &notice( "Update sent" . ( $away ? " (and away msg set)" : "" ) );
+    } else {
+      &notice( "Update sent" );
+    }
+}
+
+sub cmd_broadcast {
+    my ( $data, $server, $win ) = @_;
+
+    my $setting = Irssi::settings_get_str("twirssi_broadcast_users");
+    my @bcast_users;
+    if ($setting) {
+      @bcast_users = split /\s*,\s*/, $setting;
+    } else {
+      @bcast_users = keys %twits;
+    }
+
+    foreach my $buser ( @bcast_users ) {
+      &cmd_tweet_as( "$buser $data", $server, $win );
+    }
 }
 
 sub cmd_reply {
@@ -296,13 +326,6 @@ sub cmd_reply {
 sub cmd_reply_as {
     my ( $data, $server, $win ) = @_;
 
-    unless ( Irssi::settings_get_bool("twirssi_track_replies") ) {
-        &notice("twirssi_track_replies is required in order to reply to "
-              . "specific tweets.  Either enable it, or just use /tweet "
-              . "\@username <text>." );
-        return;
-    }
-
     return unless &logged_in($twit);
 
     $data =~ s/^\s+|\s+$//;
@@ -340,6 +363,10 @@ sub cmd_reply_as {
 
     return if &too_long($data);
 
+    if (!utf8::is_utf8($data)) {
+        $data = decode &get_charset, $data;
+    }
+
     my $success = 1;
     eval {
         unless (
@@ -362,8 +389,8 @@ sub cmd_reply_as {
         return;
     }
 
-    foreach ( $data =~ /@([-\w]+)/ ) {
-        $nicks{$1} = time;
+    foreach ( $data =~ /@([-\w]+)/g ) {
+        $nicks{$_} = time;
     }
 
     my $away = &update_away($data);
@@ -372,13 +399,17 @@ sub cmd_reply_as {
 }
 
 sub gen_cmd {
-    my ( $usage_str, $api_name, $post_ref ) = @_;
+    my ( $usage_str, $api_name, $post_ref, $data_ref ) = @_;
 
     return sub {
         my ( $data, $server, $win ) = @_;
 
         return unless &logged_in($twit);
 
+        if ($data_ref) {
+          $data = $data_ref->($data);
+        }
+
         $data =~ s/^\s+|\s+$//;
         unless ($data) {
             &notice("Usage: $usage_str");
@@ -396,7 +427,7 @@ sub gen_cmd {
         return unless $success;
 
         if ($@) {
-            &notice("$api_name caused an error.  Aborted.");
+            &notice("$api_name caused an error.  Aborted: $@");
             return;
         }
 
@@ -525,9 +556,10 @@ sub cmd_login {
             } else {
                 $twit = Net::Twitter->new(
                     traits => [ 'API::REST', 'OAuth', 'API::Search' ],
-                    consumer_key => 'BZVAvBma4GxdiRwXIvbnw',
-                    consumer_secret =>
-                      '0T5kahwLyb34vciGZsgkA9lsjtGCQ05vxVE2APXM',
+                    ( grep tr/a-zA-Z/n-za-mN-ZA-M/, map $_,
+                      pbafhzre_xrl => 'OMINiOzn4TkqvEjKVioaj',
+                      pbafhzre_frperg => '0G5xnujYlo34ipvTMftxN9yfwgTPD05ikIR2NCKZ',
+                    ),
                     source => "twirssi",
                     ssl    => !Irssi::settings_get_bool("twirssi_avoid_ssl"),
                 );
@@ -554,14 +586,14 @@ sub cmd_login {
                 eval { $url = $twit->get_authorization_url; };
 
                 if ($@) {
-                    &notice( "ERROR: Failed to get OAuth authorization_url. " .
-                             "Try again later.");
+                    &notice("ERROR: Failed to get OAuth authorization_url: $@" );
                     return;
                 }
-                &notice( "Twirssi not autorized to access $service for $user.",
-                         "Please authorize at the following url, then enter the pin",
-                         "supplied with /twirssi_oauth $user\@$service <pin>",
-                         $url
+                &notice(
+                    "Twirssi not autorized to access $service for $user.",
+                    "Please authorize at the following url, then enter the pin",
+                    "supplied with /twirssi_oauth $user\@$service <pin>",
+                    $url
                 );
 
                 $oauth{pending}{"$user\@$service"} = $twit;
@@ -690,7 +722,7 @@ sub verify_twitter_object {
     $poll = Irssi::timeout_add( &get_poll_time * 1000, \&get_updates, "" );
     &notice("Logged in as $user\@$service, loading friends list...");
     &load_friends();
-    &notice( "loaded friends: ". scalar keys %friends );
+    &notice( "loaded friends: " . scalar keys %friends );
     if ( Irssi::settings_get_bool("twirssi_first_run") ) {
         Irssi::settings_set_bool( "twirssi_first_run", 0 );
     }
@@ -991,7 +1023,7 @@ sub get_updates {
     return unless &logged_in($twit);
 
     my ( $fh, $filename ) = File::Temp::tempfile();
-    binmode( $fh, ":utf8" );
+    binmode( $fh, ":" . &get_charset );
     $child_pid = fork();
 
     if ($child_pid) {    # parent
@@ -1010,7 +1042,7 @@ sub get_updates {
         foreach ( keys %twits ) {
             $error++ unless &do_updates( $fh, $_, $twits{$_}, \%context_cache );
 
-            if ( $id_map{__fixreplies}{$_} ) {
+            if ( exists $id_map{__fixreplies}{$_} and keys %{ $id_map{__fixreplies}{$_} } ) {
                 my @frusers = sort keys %{ $id_map{__fixreplies}{$_} };
 
                 $error++
@@ -1074,13 +1106,22 @@ sub do_updates {
     print scalar localtime, " - Polling for updates for $username" if &debug;
     my $tweets;
     my $new_poll_id = 0;
+    my @ignored_accounts =
+      Irssi::settings_get_str("twirssi_ignored_accounts")
+      ? split /\s*,\s*/, Irssi::settings_get_str("twirssi_ignored_accounts")
+      : ();
     eval {
-        if ( $id_map{__last_id}{$username}{timeline} )
-        {
-            $tweets = $obj->home_timeline( { count => 100 } );
-        } else {
-            $tweets = $obj->home_timeline();
-        }
+	if ( grep { $_ eq $username } @ignored_accounts ) {
+	    $tweets = ();
+	    print $fh "type:debug Ignoring timeline for $username\n" if &debug;
+	} else {
+	    if ( $id_map{__last_id}{$username}{timeline} )
+	    {
+		$tweets = $obj->home_timeline( { count => 100 } );
+	    } else {
+		$tweets = $obj->home_timeline();
+	    }
+	}
     };
 
     if ($@) {
@@ -1105,9 +1146,31 @@ sub do_updates {
         return undef;
     }
 
+    my @ignore_tags =
+      Irssi::settings_get_str("twirssi_ignored_tags")
+      ? split /\s*,\s*/, Irssi::settings_get_str("twirssi_ignored_tags")
+      : ();
+    my @strip_tags =
+      Irssi::settings_get_str("twirssi_stripped_tags")
+      ? split /\s*,\s*/, Irssi::settings_get_str("twirssi_stripped_tags")
+      : ();
     foreach my $t ( reverse @$tweets ) {
         my $text = &get_text( $t, $obj );
         my $reply = "tweet";
+
+        my $match = 0;
+        foreach my $tag (@ignore_tags) {
+            next unless $text =~ /\b\Q$tag\E\b/i;
+            $match = 1;
+            $text = "(ignored: $tag) $text" if &debug;
+            last;
+        }
+        next if not &debug and $match;
+
+        foreach my $tag (@strip_tags) {
+            $text =~ s/(?:\b|^)\Q$tag\E(?:\b|$)//gi;
+        }
+
         if (    Irssi::settings_get_bool("show_reply_context")
             and $t->{in_reply_to_screen_name} ne $username
             and $t->{in_reply_to_screen_name}
@@ -1228,6 +1291,7 @@ sub do_updates {
             }
 
             $id_map{__searches}{$username}{$topic} = $search->{max_id};
+            $topic =~ s/ /%20/g;
             printf $fh "id:%s account:%s type:searchid topic:%s\n",
               $search->{max_id}, $username, $topic;
 
@@ -1330,7 +1394,7 @@ sub monitor_child {
     # pretend
 
     if ( open FILE, $filename ) {
-        binmode FILE, ":utf8";
+        binmode FILE, ":" . &get_charset;
         my @lines;
         my %new_cache;
         while (<FILE>) {
@@ -1344,8 +1408,9 @@ sub monitor_child {
             my %meta;
 
             foreach my $key (qw/id account nick type topic/) {
-                if (s/^$key:(\S+)\s*//) {
+                if (s/^$key:((?:\S|\\ )+)\s*//) {
                     $meta{$key} = $1;
+                    $meta{$key} =~ s/%20/ /g;
                 }
             }
 
@@ -1383,11 +1448,7 @@ sub monitor_child {
             }
 
             my $marker = "";
-            if (    $meta{type} ne 'dm'
-                and Irssi::settings_get_bool("twirssi_track_replies")
-                and $meta{nick}
-                and $meta{id} )
-            {
+            if ( $meta{type} ne 'dm' and $meta{nick} and $meta{id} ) {
                 $marker = ( $id_map{__indexes}{ $meta{nick} } + 1 ) % 100;
                 $id_map{ lc $meta{nick} }[$marker]           = $meta{id};
                 $id_map{__indexes}{ $meta{nick} }            = $marker;
@@ -1524,7 +1585,7 @@ sub monitor_child {
             Irssi::pidwait_remove($child_pid);
 
             # and that we don't leave any zombies behind, somehow
-            wait();
+            waitpid( -1, WNOHANG );
 
             # save id_map hash
             if ( keys %id_map
@@ -1552,7 +1613,7 @@ sub monitor_child {
     } else {
         print "Giving up on polling $filename" if &debug;
         Irssi::pidwait_remove($child_pid);
-        wait();
+        waitpid( -1, WNOHANG );
         unlink $filename unless &debug;
 
         return unless Irssi::settings_get_bool("twirssi_notify_timeouts");
@@ -1575,7 +1636,7 @@ sub monitor_child {
                 q{      \\          a    |},
                 q{       ',.__.   ,__.-'/},
                 q{         '--/_.'----'`}
-              );
+            );
             $failwhale = 1;
         }
 
@@ -1663,7 +1724,7 @@ sub sig_complete {
     my ( $complist, $window, $word, $linestart, $want_space ) = @_;
 
     if (
-        $linestart =~ /^\/(?:retweet|twitter_reply)(?:_as)?\s*$/
+        $linestart =~ m{^/twitter_delete\s*$|^/(?:retweet|twitter_reply)(?:_as)?\s*$}
         or ( Irssi::settings_get_bool("twirssi_use_reply_aliases")
             and $linestart =~ /^\/reply(?:_as)?\s*$/ )
       )
@@ -1714,6 +1775,12 @@ sub get_poll_time {
     return 60;
 }
 
+sub get_charset {
+    my $charset = Irssi::settings_get_str("twirssi_charset");
+    return "utf8" if $charset =~ /^\s*$/;
+    return $charset;
+}
+
 sub hilight {
     my $text = shift;
 
@@ -1754,7 +1821,11 @@ sub shorten {
                     "Set short_url_args to username,API_key or change your",
                     "short_url_provider."
                 );
-                return decode "utf8", $data;
+                if (!utf8::is_utf8($data)) {
+                    return decode &get_charset, $data;
+                } else {
+                    return $data;
+                }
             }
         }
 
@@ -1771,7 +1842,11 @@ sub shorten {
         }
     }
 
-    return decode "utf8", $data;
+    if (!utf8::is_utf8($data)) {
+        return decode &get_charset, $data;
+    } else {
+        return $data;
+    }
 }
 
 sub normalize_username {
@@ -1832,15 +1907,20 @@ Irssi::theme_register(
 );
 
 Irssi::settings_add_int( "twirssi", "twitter_poll_interval", 300 );
+Irssi::settings_add_str( "twirssi", "twirssi_charset",         "utf8" );
 Irssi::settings_add_str( "twirssi", "twitter_window",          "twitter" );
 Irssi::settings_add_str( "twirssi", "bitlbee_server",          "bitlbee" );
 Irssi::settings_add_str( "twirssi", "short_url_provider",      "TinyURL" );
 Irssi::settings_add_str( "twirssi", "short_url_args",          undef );
 Irssi::settings_add_str( "twirssi", "twitter_usernames",       undef );
 Irssi::settings_add_str( "twirssi", "twitter_passwords",       undef );
+Irssi::settings_add_str( "twirssi", "twirssi_broadcast_users", undef );
 Irssi::settings_add_str( "twirssi", "twirssi_default_service", "Twitter" );
 Irssi::settings_add_str( "twirssi", "twirssi_nick_color",      "%B" );
 Irssi::settings_add_str( "twirssi", "twirssi_topic_color",     "%r" );
+Irssi::settings_add_str( "twirssi", "twirssi_ignored_tags",    "" );
+Irssi::settings_add_str( "twirssi", "twirssi_stripped_tags",   "" );
+Irssi::settings_add_str( "twirssi", "twirssi_ignored_accounts","" );
 Irssi::settings_add_str( "twirssi", "twirssi_retweet_format",
     'RT $n: "$t" ${-- $c$}' );
 Irssi::settings_add_str( "twirssi", "twirssi_location",
@@ -1859,7 +1939,6 @@ Irssi::settings_add_bool( "twirssi", "show_reply_context",        0 );
 Irssi::settings_add_bool( "twirssi", "show_own_tweets",           1 );
 Irssi::settings_add_bool( "twirssi", "twirssi_debug",             0 );
 Irssi::settings_add_bool( "twirssi", "twirssi_first_run",         1 );
-Irssi::settings_add_bool( "twirssi", "twirssi_track_replies",     1 );
 Irssi::settings_add_bool( "twirssi", "twirssi_replies_autonick",  1 );
 Irssi::settings_add_bool( "twirssi", "twirssi_use_reply_aliases", 0 );
 Irssi::settings_add_bool( "twirssi", "twirssi_notify_timeouts",   1 );
@@ -1889,6 +1968,7 @@ if ($window) {
     Irssi::command_bind( "tweet_as",                   "cmd_tweet_as" );
     Irssi::command_bind( "retweet",                    "cmd_retweet" );
     Irssi::command_bind( "retweet_as",                 "cmd_retweet_as" );
+    Irssi::command_bind( "twitter_broadcast",          "cmd_broadcast" );
     Irssi::command_bind( "twitter_reply",              "cmd_reply" );
     Irssi::command_bind( "twitter_reply_as",           "cmd_reply_as" );
     Irssi::command_bind( "twitter_login",              "cmd_login" );
@@ -1946,6 +2026,18 @@ if ($window) {
             );
         }
     );
+    Irssi::command_bind(
+        "twitter_delete",
+        &gen_cmd(
+            "/twitter_delete <username:id>",
+            "destroy_status",
+            sub { &notice("Tweet deleted."); },
+            sub { my ($nick, $num) = split /:/, lc $_[0], 2;
+                  $num = $id_map{__last_tweet}{&normalize_username($nick)} 
+                    unless (defined $num);
+                  return $id_map{$nick}[$num]; } 
+        )
+    );
     Irssi::command_bind(
         "twitter_follow",
         &gen_cmd(
@@ -1986,11 +2078,21 @@ if ($window) {
             sub { &notice("Unblock $_[0]"); }
         )
     );
+    Irssi::command_bind(
+        "twitter_spam",
+        &gen_cmd(
+            "/twitter_spam <username>",
+            "report_spam",
+            sub { &notice("Reported $_[0] for spam"); }
+        )
+    );
     Irssi::signal_add_last( 'complete word' => \&sig_complete );
 
-    &notice("  %Y<%C(%B^%C)%N                   TWIRSSI v%R$VERSION%N",
-            "   %C(_(\\%N           http://twirssi.com/ for full docs",
-            "    %Y||%C `%N Log in with /twitter_login, send updates with /tweet");
+    &notice(
+        "  %Y<%C(%B^%C)%N                   TWIRSSI v%R$VERSION%N",
+        "   %C(_(\\%N           http://twirssi.com/ for full docs",
+        "    %Y||%C `%N Log in with /twitter_login, send updates with /tweet"
+    );
 
     my $file = Irssi::settings_get_str("twirssi_replies_store");
     if ( $file and -r $file ) {
